import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

import '../../../get_core/get_core.dart';
import '../../../get_state_manager/get_state_manager.dart';
import '../../../get_utils/get_utils.dart';
import '../../get_navigation.dart';

class GetMaterialApp extends StatefulWidget {
  final GlobalKey<NavigatorState>? navigatorKey;
  final GlobalKey<ScaffoldMessengerState>? scaffoldMessengerKey;
  final Widget? home;
  final Map<String, WidgetBuilder>? routes;
  final String? initialRoute;
  final RouteFactory? onGenerateRoute;
  final InitialRouteListFactory? onGenerateInitialRoutes;
  final RouteFactory? onUnknownRoute;
  final List<NavigatorObserver>? navigatorObservers;
  final TransitionBuilder? builder;
  final String title;
  final GenerateAppTitle? onGenerateTitle;
  final ThemeData? theme;
  final ThemeData? darkTheme;
  final ThemeMode themeMode;
  final CustomTransition? customTransition;
  final Color? color;
  final Map<String, Map<String, String>>? translationsKeys;
  final Translations? translations;
  final TextDirection? textDirection;
  final Locale? locale;
  final Locale? fallbackLocale;
  final Iterable<LocalizationsDelegate<dynamic>>? localizationsDelegates;
  final LocaleListResolutionCallback? localeListResolutionCallback;
  final LocaleResolutionCallback? localeResolutionCallback;
  final Iterable<Locale> supportedLocales;
  final bool showPerformanceOverlay;
  final bool checkerboardRasterCacheImages;
  final bool checkerboardOffscreenLayers;
  final bool showSemanticsDebugger;
  final bool debugShowCheckedModeBanner;
  final Map<LogicalKeySet, Intent>? shortcuts;
  final ScrollBehavior? scrollBehavior;
  final ThemeData? highContrastTheme;
  final ThemeData? highContrastDarkTheme;
  final Map<Type, Action<Intent>>? actions;
  final bool debugShowMaterialGrid;
  final ValueChanged<Routing?>? routingCallback;
  final Transition? defaultTransition;
  final bool? opaqueRoute;
  final VoidCallback? onInit;
  final VoidCallback? onReady;
  final VoidCallback? onDispose;
  final bool? enableLog;
  final LogWriterCallback? logWriterCallback;
  final bool? popGesture;
  final SmartManagement smartManagement;
  final List<Bind> binds;
  final Duration? transitionDuration;
  final bool? defaultGlobalState;
  final List<GetPage>? getPages;
  final GetPage? unknownRoute;
  final RouteInformationProvider? routeInformationProvider;
  final RouteInformationParser<Object>? routeInformationParser;
  final RouterDelegate<Object>? routerDelegate;
  final BackButtonDispatcher? backButtonDispatcher;
  final bool useInheritedMediaQuery;

  GetMaterialApp({
    Key? key,
    this.navigatorKey,
    this.scaffoldMessengerKey,
    this.home,
    Map<String, Widget Function(BuildContext)> this.routes =
        const <String, WidgetBuilder>{},
    this.initialRoute,
    this.onGenerateRoute,
    this.onGenerateInitialRoutes,
    this.onUnknownRoute,
    this.useInheritedMediaQuery = false,
    List<NavigatorObserver> this.navigatorObservers =
        const <NavigatorObserver>[],
    this.builder,
    this.textDirection,
    this.title = '',
    this.onGenerateTitle,
    this.color,
    this.theme,
    this.darkTheme,
    this.themeMode = ThemeMode.system,
    this.locale,
    this.fallbackLocale,
    this.localizationsDelegates,
    this.localeListResolutionCallback,
    this.localeResolutionCallback,
    this.supportedLocales = const <Locale>[Locale('en', 'US')],
    this.debugShowMaterialGrid = false,
    this.showPerformanceOverlay = false,
    this.checkerboardRasterCacheImages = false,
    this.checkerboardOffscreenLayers = false,
    this.showSemanticsDebugger = false,
    this.debugShowCheckedModeBanner = true,
    this.shortcuts,
    this.scrollBehavior,
    this.customTransition,
    this.translationsKeys,
    this.translations,
    this.onInit,
    this.onReady,
    this.onDispose,
    this.routingCallback,
    this.defaultTransition,
    this.getPages,
    this.opaqueRoute,
    this.enableLog = kDebugMode,
    this.logWriterCallback,
    this.popGesture,
    this.transitionDuration,
    this.defaultGlobalState,
    this.smartManagement = SmartManagement.full,
    this.binds = const [],
    this.unknownRoute,
    this.highContrastTheme,
    this.highContrastDarkTheme,
    this.actions,
  })  : routeInformationProvider = null,
        backButtonDispatcher = null,
        routeInformationParser = null,
        routerDelegate = null,
        super(key: key);

  static String _cleanRouteName(String name) {
    name = name.replaceAll('() => ', '');

    /// uncommonent for URL styling.
    // name = name.paramCase!;
    if (!name.startsWith('/')) {
      name = '/$name';
    }
    return Uri.tryParse(name)?.toString() ?? name;
  }

  GetMaterialApp.router({
    Key? key,
    this.routeInformationProvider,
    this.scaffoldMessengerKey,
    this.routeInformationParser,
    this.routerDelegate,
    this.backButtonDispatcher,
    this.builder,
    this.title = '',
    this.onGenerateTitle,
    this.color,
    this.theme,
    this.darkTheme,
    this.useInheritedMediaQuery = false,
    this.highContrastTheme,
    this.highContrastDarkTheme,
    this.themeMode = ThemeMode.system,
    this.locale,
    this.localizationsDelegates,
    this.localeListResolutionCallback,
    this.localeResolutionCallback,
    this.supportedLocales = const <Locale>[Locale('en', 'US')],
    this.debugShowMaterialGrid = false,
    this.showPerformanceOverlay = false,
    this.checkerboardRasterCacheImages = false,
    this.checkerboardOffscreenLayers = false,
    this.showSemanticsDebugger = false,
    this.debugShowCheckedModeBanner = true,
    this.shortcuts,
    this.scrollBehavior,
    this.actions,
    this.customTransition,
    this.translationsKeys,
    this.translations,
    this.textDirection,
    this.fallbackLocale,
    this.routingCallback,
    this.defaultTransition,
    this.opaqueRoute,
    this.onInit,
    this.onReady,
    this.onDispose,
    this.enableLog = kDebugMode,
    this.logWriterCallback,
    this.popGesture,
    this.smartManagement = SmartManagement.full,
    this.binds = const [],
    this.transitionDuration,
    this.defaultGlobalState,
    this.getPages,
    this.navigatorObservers,
    this.unknownRoute,
  })  : navigatorKey = null,
        onGenerateRoute = null,
        home = null,
        onGenerateInitialRoutes = null,
        onUnknownRoute = null,
        routes = null,
        initialRoute = null,
        super(key: key);

  @override
  State<GetMaterialApp> createState() => _GetMaterialAppState();
}

class _GetMaterialAppState extends State<GetMaterialApp> {
  late final RouterDelegate<Object> routerDelegate;
  late final RouteInformationParser<Object> routeInformationParser;

  @override
  void initState() {
    routerDelegate = widget.routerDelegate ??
        Get.createDelegate(
          pages: widget.getPages ?? [],
          notFoundRoute: widget.unknownRoute,
          navigatorKey: widget.navigatorKey,
          navigatorObservers: (widget.navigatorObservers == null
              ? <NavigatorObserver>[
                  GetObserver(widget.routingCallback, Get.routing)
                ]
              : <NavigatorObserver>[
                  GetObserver(widget.routingCallback, Get.routing),
                  ...widget.navigatorObservers!
                ]),
        );

    routeInformationParser = widget.routeInformationParser ??
        Get.createInformationParser(
          initialRoute: widget.initialRoute ??
              widget.getPages?.first.name ??
              GetMaterialApp._cleanRouteName("/${widget.home.runtimeType}"),
        );

    Get.engine.addPostFrameCallback((timeStamp) => widget.onReady?.call());

    if (widget.locale != null) Get.locale = widget.locale;

    if (widget.fallbackLocale != null) {
      Get.fallbackLocale = widget.fallbackLocale;
    }

    if (widget.translations != null) {
      Get.addTranslations(widget.translations!.keys);
    } else if (widget.translationsKeys != null) {
      Get.addTranslations(widget.translationsKeys!);
    }

    Get.customTransition = widget.customTransition;

    if (widget.getPages != null) {
      Get.addPages(widget.getPages!);
    } else {
      Get.addPage(
        GetPage(
          name: GetMaterialApp._cleanRouteName("/${widget.home.runtimeType}"),
          page: () => widget.home!,
        ),
      );
    }

    //Get.setDefaultDelegate(routerDelegate);
    Get.smartManagement = widget.smartManagement;
    widget.onInit?.call();

    Get.config(
      enableLog: widget.enableLog ?? Get.isLogEnable,
      logWriterCallback: widget.logWriterCallback,
      defaultTransition: widget.defaultTransition ?? Get.defaultTransition,
      defaultOpaqueRoute: widget.opaqueRoute ?? Get.isOpaqueRouteDefault,
      defaultPopGesture: widget.popGesture ?? Get.isPopGestureEnable,
      defaultDurationTransition:
          widget.transitionDuration ?? Get.defaultTransitionDuration,
    );
    super.initState();
  }

  @override
  void dispose() {
    widget.onDispose?.call();
    Get.clearRouteTree();
    Get.clearTranslations();
    Get.resetRootNavigator();
    Get.routerDelegate = null;
    Get.routeInformationParser = null;
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Binds(
      binds: [
        Bind.put(Get.rootController),
        ...widget.binds,
      ],
      child: Builder(builder: (context) {
        final controller = context.listen<GetMaterialController>();
        return MaterialApp.router(
          routerDelegate: routerDelegate,
          routeInformationParser: routeInformationParser,
          backButtonDispatcher: widget.backButtonDispatcher,
          routeInformationProvider: widget.routeInformationProvider,
          key: controller.unikey,
          builder: (context, child) => Directionality(
            textDirection: widget.textDirection ??
                (rtlLanguages.contains(Get.locale?.languageCode)
                    ? TextDirection.rtl
                    : TextDirection.ltr),
            child: widget.builder == null
                ? (child ?? Material())
                : widget.builder!(context, child ?? Material()),
          ),
          title: widget.title,
          onGenerateTitle: widget.onGenerateTitle,
          color: widget.color,
          theme: controller.theme ?? widget.theme ?? ThemeData.fallback(),
          darkTheme: controller.darkTheme ??
              widget.darkTheme ??
              widget.theme ??
              ThemeData.fallback(),
          themeMode: controller.themeMode ?? widget.themeMode,
          locale: Get.locale ?? widget.locale,
          scaffoldMessengerKey:
              widget.scaffoldMessengerKey ?? controller.scaffoldMessengerKey,
          localizationsDelegates: widget.localizationsDelegates,
          localeListResolutionCallback: widget.localeListResolutionCallback,
          localeResolutionCallback: widget.localeResolutionCallback,
          supportedLocales: widget.supportedLocales,
          debugShowMaterialGrid: widget.debugShowMaterialGrid,
          showPerformanceOverlay: widget.showPerformanceOverlay,
          checkerboardRasterCacheImages: widget.checkerboardRasterCacheImages,
          checkerboardOffscreenLayers: widget.checkerboardOffscreenLayers,
          showSemanticsDebugger: widget.showSemanticsDebugger,
          debugShowCheckedModeBanner: widget.debugShowCheckedModeBanner,
          shortcuts: widget.shortcuts,
          scrollBehavior: widget.scrollBehavior,
          useInheritedMediaQuery: widget.useInheritedMediaQuery,
        );
      }),
    );
  }
}
